EPGPreserves = LibStub("AceAddon-3.0"):NewAddon("EPGPreserves", "AceConsole-3.0", "AceEvent-3.0","AceHook-3.0")

epgpraidhandling = {}
epgpreserveInProgress = false 

local defaults = {
    profile = {
        reservists={},
		Popuppoint = "CENTER", -- remember popup window position
		PopuprelativeTo = nil, 
		PopuprelativePoint = "CENTER", 
		PopuppointxOfs = 0, 
		PopuppointyOfs = 0,
    },
}

local options = {
    name = "EPGPreservist",
    handler = EPGPreserves,
	order = 1,
    type = "group",
    args = {
	
		clearList = {
			name = "clearlist", type = "execute",
			desc = "Clear the reservist list.",
			func = "clearListCMD",
			guiHidden= true,
			dialogHidden = true,
			dropdownHidden = true, 
		},	
		showList = {
			name = "showlist", type = "execute",
			desc = "Show the reservist list.",
			func = "showListCMD",
			guiHidden= true,
			dialogHidden = true,
			dropdownHidden = true, 
		},	
		add = {
			name = "add", type = "input",
			desc = "Add someone to the list.",
			set = "AddReservistCMD",
			guiHidden= true,
			dialogHidden = true,
			dropdownHidden = true, 
		},	
		remove = {
			name = "remove", type = "input",
			desc = "Remove someone to the list.",
			set = "RemoveReservistCMD",
			guiHidden= true,
			dialogHidden = true,
			dropdownHidden = true, 
		},	
		announce = {
			name = "announce", type = "input",
			desc = "Announce the reservist list in a channel (default: guild).",
			set = "AnnounceListCMD",
			guiHidden= true,
			dialogHidden = true,
			dropdownHidden = true, 
		},	
		raidend = {
			name = "raidend", type = "execute",
			desc = "Show popups for everyone in the list.",
			func = "raidendCMD",
			guiHidden= true,
			dialogHidden = true,
			dropdownHidden = true, 
		},	
		award = {
			name = "award", type = "input",
			desc = "Award the reserve EP to the person.",
			set = "AwardListCMD",
			guiHidden= true,
			dialogHidden = true,
			dropdownHidden = true, 
		},	
	},
}

function EPGPreserves:AddReservistCMD(info,value)
	if value=="" or value==nil then
		self:Print("You must specify a name.")
	else
		local name = string.upper(string.sub(value,0,1))..string.lower(string.sub(value,2))
		self.db.profile.reservists[name]=0
		self:Print("You added "..name.." to the list.")
	end
end

function EPGPreserves:AwardListCMD(info,value)
	if value=="" or value==nil then
		self:Print("You must specify a name.")
	else
		local name = string.upper(string.sub(value,0,1))..string.lower(string.sub(value,2))
		if (self.db.profile.reservists[name]==nil) then
			self:Print(name.." is not in the list.")
		else
			self:AwardEP(name)
		end
	end
end

function EPGPreserves:RemoveReservistCMD(info,value)
	if value=="" or value==nil then
		self:Print("You must specify a name.")
	else
		local name = string.upper(string.sub(value,0,1))..string.lower(string.sub(value,2))
		if (self.db.profile.reservists[name]==nil) then
			self:Print(name.." is not in the list.")
		else
			self.db.profile.reservists[name]=nil
			self:Print(name.." is removed from the list.")
		end
	end
end


function EPGPreserves:AnnounceListCMD(info,channelvalue)
	if channelvalue=="" or channelvalue==nil then
		channelvalue="GUILD"
	end
	
	channelvalue=string.upper(channelvalue)
	if (channelvalue~="GUILD")and(channelvalue~="RAID")and(channelvalue~="OFFICER") then
		self:Print("Please choose one of the following channels: guild, raid, officer")
		return
	end

	SendChatMessage("Current reservists:" , channelvalue, nil)
	local i=0
	for key,value in pairs(self.db.profile.reservists) do
		i=i+1
		SendChatMessage(key.." ("..self.db.profile.reservists[key].." reserve EP)" , channelvalue, nil)
	end
	if(i==0)then
		SendChatMessage("-no reservists" , channelvalue, nil)
	end
	
end

function EPGPreserves:clearListCMD()
	self.db.profile.reservists={}
	self:Print("The list of reservists is empty.")
end

function EPGPreserves:showListCMD()

	for key,value in pairs(self.db.profile.reservists) do
		self:Print(key.." "..self.db.profile.reservists[key])
	end
end

function EPGPreserves:raidendCMD()

	self:Print("Showing award dialogs.")
	for key,value in pairs(self.db.profile.reservists) do
		epgpraidhandling[key]=true
		self:Print("Added "..key)
	end
	self:DoNextPopup()
		
end


function EPGPreserves:OnInitialize()
    -- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("EPGPreservesDB", defaults, "Default")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("EPGPreservist", options, {"EPGPreservist","epgpr"})
--	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EPGPreservist", "EPGPreservist")


end


function EPGPreserves:OnEnable()
	self:Print("EPGPreserves 1.02 loaded.")
    EPGP.RegisterCallback(self, "MassEPAward")
	
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_OFFICER")
end
 
function EPGPreserves:MassEPAward(event, awarded, reason, amount)

	for key,value in pairs(self.db.profile.reservists) do
		--Check if the player has been awarded anything via the mass award. If so; don't update the value in this list.
		if awarded[key]==nil then
			--that player didn't get anything. Update the reservist value:
			--self:Print("Awarding reserve EP to "..key)
			self.db.profile.reservists[key]=value+amount
		end
	end
	
end

function EPGPreserves:AwardEP(name)
	EPGP:IncEPBy(name,"Reserve duty",self.db.profile.reservists[name])
	SendChatMessage("[EPGPreserve]: You are now removed from the reserve list. Awarded "..self.db.profile.reservists[name].." reserve EP.", "WHISPER", nil,name)		
	self.db.profile.reservists[name]=nil
end

function EPGPreserves:RemoveFromList(name)
	self:Print("You remove "..name.." from the list.")
	self.db.profile.reservists[name]=nil
end

function EPGPreserves:RAID_ROSTER_UPDATE()
	--figure out which member joined the raid
	--self:Print("Roster updated")
	
	for i = 1,GetNumRaidMembers() do
		name, _, _, _, _, _, _, _, _, _, _ = GetRaidRosterInfo(i)
		if self.db.profile.reservists[name]~=nil then
			--this player is on the reservelist AND is in this raid.
			if epgpraidhandling[name]~=true and self.db.profile.reservists[name]>0 then
				--self:Print(name.." is in the raid and has: "..self.db.profile.reservists[name].." reserve EP.")
				epgpraidhandling[name]=true
			else
				--self:Print("You already have a popup dialog open for "..name..".")
			end
		end
	end
	if(epgpreserveInProgress == false)then
		self:DoNextPopup()
	end
end

function EPGPreserves:CHAT_MSG_ADDON(event,arg1,arg2,arg3,arg4)
	if (arg1 == "EPGPreserves") then
	--This message is for me! Handle it!
	end
end

local function IsPlayerInMyGuild(targetplayer)
	local total = GetNumGuildMembers(true)
	for i = 1, total 
	   do 
	   -- get info
	   local name, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName = GetGuildRosterInfo(i)
			if name==targetplayer then
				return true
			end
	   end
	return false
end

function EPGPreserves:CHAT_MSG_OFFICER(event,arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11)

	if arg1=="reservelist" or arg1=="Reservelist" or arg1=="Who is on reserve?" or arg1=="who is reserve?" then
		SendChatMessage("Current reservists:" , "OFFICER", nil)
		local i=0
		for key,value in pairs(self.db.profile.reservists) do
			i=i+1
			SendChatMessage(key.." ("..self.db.profile.reservists[key].." reserve EP)" , "OFFICER", nil)
		end
		if(i==0)then
			SendChatMessage("-no reservists" , "OFFICER", nil)
		end		
	end


end

function EPGPreserves:CHAT_MSG_WHISPER(event,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8)

--arg1 
--Message received 
--arg2 
--Author 
--arg3 
--Language (or nil if universal, like messages from GM) (always seems to be an empty string; argument may have been kicked because whispering in non-standard language doesn't seem to be possible [any more?]) 
--arg6 
--status (like "DND" or "GM") 
--arg7 
--(number) message id (for reporting spam purposes?) (default: 0) 
--arg8 
--(number) unknown (default: 0) 

	if (string.lower(arg1)=="reserve") then
	--This person wants to be reserve.
		--check if the player is in the same guild as the person who whispered:
		
		if IsPlayerInMyGuild(arg2) then
			--Okay this is a serious player.
			
			--Check if this player isn't already on my list.
			if self.db.profile.reservists[arg2]~=nil then

				SendChatMessage("[EPGPreserve]: You are already registered as reservist; your reservist EP is: "..self.db.profile.reservists[arg2], "WHISPER", nil,arg2)		
			else

				--This player is not yet on the list.
				--Add the player to the list:
				self.db.profile.reservists[arg2]=0
				--Notify the player
				SendChatMessage("[EPGPreserve]: You are now registered as reservist; be sure to be on standby.", "WHISPER", nil,arg2)		
				--Notify the rest of the people tracking reservists:
				SendAddonMessage("EPGPreserves","added reservist", "RAID")
			end
		
		else
			SendChatMessage("[EPGPreserve]: I cannot put you on the reserve list; you're not in my guild.", "WHISPER", nil,arg2)		
		end
	
	end
	
	if arg1=="reservelist" then
		local strNames = ""
		for key,value in pairs(self.db.profile.reservists) do
			strNames = strNames.." "..key.."("..value.."EP)"
		end
		SendChatMessage("[EPGPreserve]: Current registered reservists: "..strNames, "WHISPER", nil,arg2)		
	end
end
